/**
 * Add image upload dialog for Batchaction buttons.
 *
 * The `.productUiVariationslideshowUnity()` method adds functionality to open an image upload dialog.
 *
 *
 * ### Examples
 * Apply .productUiVariationslideshowUnity() to Button element.
 *
 * JavaScript:
 *
 *      require([
 *        'jquery',
 *        'de_epages/product/ui/variationslideshow-unity',
 *
 *        'jquery/metaparse',
 *        '$ready!'
 *      ], function ($, ep) {
 *        ep('#ep-variationslideshow-button').on('click', function() {
 *          var self = ep(this);
 *
 *          self.off('click');
 *          self.productUiVariationslideshowUnity({
 *            items: function() {
 *                return [{
 *                  "value": #ObjectID
 *                }];
 *            },
 *            dndUploader: {
 *                #IF(#DEFINED(#WebInterfaceConf.MAXCONTENTLENGTH))maxFileSize: #WebInterfaceConf.MAXCONTENTLENGTH,#ENDIF
 *                #IF(#DEFINED(#Exist))
 *                    exist: JSON.parse('#AllExist[0]'),
 *                    order: #AllOrder[0],
 *                #ENDIF
 *            },
 *            action: 'UnityMBO-SetVariationSlideshow',
 *            viewAction: 'UnityMBO-ViewGeneral',
 *            objectID: #ObjectID,
 *            productName: '#Name'
 *            #IF(#DEFINED(#SlideShowOrder)), slideshowArrays: JSON.parse('#SlideShowOrder[0]')#ENDIF
 *          });
 *        });
 *      });
 *
 * HTML:
 *
 *     <button>Click Me!</button>
 *
 *
 * @class jQuery.ui.productUiVariationslideshowUnity
 *
 * @uses de_epages
 * @uses de_epages.presentation.ui.dnduploader
 * @uses ep
 * @uses ep.ui.dialog
 * @uses jquery.ui.widget
 * @since 7.4.0
 */

/**
 * @cfg {Number} objectId ObjectId of the master product
 * @cfg {String} action Name of the Changeaction for submitting the image
 * @cfg {String} ViewAction Name of the ViewAction to display after submitting the image
 * @cfg {String} DeleteAction Name of the Changeaction for deleting the image
 * @cfg {String} productName Name of Master Product
 * @cfg {Object} slideshowArrays Object with "Subproduct -> array of slideshow images"-mapping
 * @cfg {Object} dndUploader config-object for dnd-uploader
 * @cfg {Function} items Function returning an Array elements whoms value is the target products id
 */

/*
 * @copyright       © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/product/ui/appointmentpicker
 */
define('de_epages/product/ui/variationslideshow-unity', [
    'jquery',
    'ep',
    '$tmpl!./variationslideshow-unity',
    '$tmpl!./variationslideshow-unity-listitem',
    '$dict!../dictionary',

    'de_epages/presentation/ui/dnduploader',
    'jquery/ui/widget',
    'ep/ui/dialog'
], function ($, ep, tmplUploadForm, tmplListItem, dict) {
    /*
     * @dictionary      ep.dict
     *
     * @translations    {Save}
     *                  {Cancel}
     *                  {UnityEditGallery}
     *
     * @dictionary      de_epages.product.dict
     *
     * @translations   {UnitySlideshowInfoText1}
     *                 {UnitySlideshowInfoText2}
     *                 {UnityNoImagesHead}
     *                 {UnityNoImagesText}
     */

    // translation object
    var translations = {
        Save: dict.translate('Save'),
        Cancel: dict.translate('Cancel'),

        editGallery: dict.translate('UnityEditGallery'),
        info1: dict.translate('UnitySlideshowInfoText1'),
        info2: dict.translate('UnitySlideshowInfoText2'),
        noImagesHead: dict.translate('UnityNoImagesHead'),
        noImagesText: dict.translate('UnityNoImagesText')
    };

    $.widget('ui.productUiVariationslideshowUnity', {
        templateElements: {},
        placeholder: {},

        options: {
            slideshowArrays: {},
            productName: 'Dummyproduct',
            action: 'myDummyChangeAction',
            viewAction: 'UnityMBO-ViewVariations',
            items: function() {
                return [];
            }
        },

        _create: function() {
            var self = this,
                elem = self.element,
                o = self.options,
                id,
                removeItemFromList;

            // load template and push named tmpl elements to current instance
            self.dialogContent = tmplUploadForm({
                ObjectId: o.objectID,
                ChangeAction: o.action,
                ViewAction: o.viewAction,
                tInfo1: translations.info1,
                tInfo2: translations.info2,
                tNoImagesText: translations.noImagesText,
                tNoImagesHead: translations.noImagesHead
            });
            $.extend(self.templateElements, self.dialogContent.tmplItem('elements'));

            // turn current images into draggables and initialize the slideshow
            self.templateElements.dndUploader.on('dnduploader:initialized', function(evt) {
                if (!o.itemIds) {
                    if (o.items().length === 0) {
                        return;
                    }
                    o.itemIds = $.makeArray(o.items()).map(function(checkbox) {
                        return checkbox.value;
                    });
                }

                self.templateElements.dndUploader.find('li').draggable({
                    revert: 'invalid',
                    connectToSortable: '.ep-variations-selectedimages > ul',
                    helper: 'clone',
                    revertDuration: 100,
                    start: $.proxy(self, "_initSorting"),
                    stop: $.proxy(self, "_stopSorting")
                });

                self.templateElements.dndUploader.find('li').addClass('fa')
                    .find('img').prop({draggable: false});

                self._initializeSlideshows();
            });

            // remove deleted images from slideshow
            self.templateElements.dndUploader.on('dnduploader:deleting', function(evt, data) {
                data.imageIds.forEach(function(imageId) {
                   self._removeImageById(imageId);
                });
            });

            // turn newly uploaded images into draggables
            self.templateElements.dndUploader.on('dnduploader:uploaded', function(evt, data) {
                var newLI = self.templateElements.dndUploader.find('li:not(.ui-draggable)');

                if (self.dialog) {
                    self.dialog.trigger('change.uiForm');
                }

                newLI.addClass('fa')
                    .find('img').prop({draggable: false});

                newLI.draggable({
                    revert: 'invalid',
                    connectToSortable: '.ep-variations-selectedimages > ul',
                    helper: 'clone',
                    revertDuration: 100,
                    start: $.proxy(self, "_initSorting"),
                    stop: $.proxy(self, "_stopSorting")
                });

                data.imageIds.forEach(function(imageId) {
                   self._selectImageById(imageId);
                });
            });

            // initialize click handler for pool images
            self.templateElements.dndUploader.on('click', 'li.ep-dropzone-sortable-item', {self: self}, self._handlePoolClick);

            // initialize click handler for slideshow images
            self.templateElements.sortableList.on('click', 'li.ep-variationslideshow-listitem', {self: self}, self._handleSelectedListClick);

            // init dndUploader
            self.templateElements.dndUploader.presentationUiDnduploader({
                newProduct: false,
                externalDropzone: self.templateElements.dropzone,
                multiple: true,
                sortable: false,
                showDescription: true,
                objectID: o.objectID,
                dropzoneFirst: true,
                showDeleteWarning: true,
                saveSlideshow: false,
                maxFileSize: o.dndUploader.maxSize,
                files: {
                    exist: o.dndUploader.exist,
                    order: o.dndUploader.order
                }
            });

            // init sortable list
            self.templateElements.sortableList.sortable({
                // options
                appendTo: document.body,
                helper: "clone",
                placeholder: "ep-dropzone-state-highlight",
                items: "li:not(ep-variationsslideshow-emptyslideshow)",
                opacity: 0.7,
                cursorAt: {
                    left: 65,
                    top: 65
                },
                tolerance: 'pointer',
                forceHelperSize: false,

                receive: $.proxy(self, "_handlePoolDrop"),
                start: function(evt, ui) {
                    self._initSorting(ui.item.is(':first-child'));
                    ui.helper.addClass('ep-variationsslideshow-helper');
                    ui.item.show();

                    ui.helper.css('margin', 'calc(' + ui.item.css('margin-top') + ' - ' + ui.helper.css('margin-top') + ') calc(' + ui.item.css('margin-left') + ' - ' + ui.helper.css('margin-left') + ')' );
                    ui.helper.addClass((parseInt(Math.random() * new Date().getTime())%1107 === 0) ? 'ep-variationsslideshow-easteregg' : '');
                    ui.item.addClass('ep-variationsslideshow-item');
                },
                beforeStop: function(evt, ui) {
                    var helperPosition = ui.helper[0].getBoundingClientRect(),
                        containerPosition = self.templateElements.dndWrapper[0].getBoundingClientRect();

                    if (helperPosition.top > containerPosition.top &&
                        helperPosition.bottom < containerPosition.bottom &&
                        helperPosition.left > containerPosition.left &&
                        helperPosition.right < containerPosition.right) {

                        removeItemFromList = true;
                    } else {
                        removeItemFromList = false;
                    }
                },
                stop: function(evt, ui) {
                    self._stopSorting();
                    ui.item.removeClass('ep-variationsslideshow-item');

                    if (removeItemFromList) {
                        self._removeImageByElement(ui.item.find('img'));
                    }
                }
            });

            // register open dialog action to batch action button
            elem.on('click', function(evt) {
                self._openDialog();
            });

            // open dialog if environment is valid
            if (self._validateEnvironment()) {
                self._openDialog();
            }
        },

        _openDialog: function() {
            var self = this,
                o = self.options,
                selectedVariations = [];

            if (!self._validateEnvironment()) {
                return;
            }

            //create array of ids of selected items
            o.itemIds = $.makeArray(o.items()).map(function(checkbox) {
                return checkbox.value;
            });

            // if not at least one item id abort
            if (o.itemIds.length === 0) {
                return;
            }

            // open if dialog exists, otherwise create new dialog
            if (self.dialog) {

                self.dialog.uiDialog('open');
                self._initializeSlideshows();

            } else {
                self.dialog = ep(self.dialogContent).uiDialog({
                    modal: true,
                    autoOpen: true,
                    width: 1120,
                    dialogClass: 'ep-variations-slideshow-dialog container-fluid',
                    title: translations.editGallery,
                    resizeable: false,
                    form: {
                        action: '?ViewAction=UnityMBO-ViewVariations&ObjectID=' + o.objectID,
                        method: 'post',
                        enctype: 'multipart/form-data'
                    },
                    close: function() {
                        self.templateElements.sortableList.find('li.ep-variationslideshow-listitem img').get().forEach($.proxy(self, "_removeImageByElement"));
                    },
                    buttons: {
                        Ok: {
                            cssClass: 'ep-button ep-button-save',
                            type: 'submit',
                            text: translations.Save,
                            click: function() {
                                var slideshowData,
                                    newSlideshowString;

                                // remove all inputs from the dndUploader
                                self.templateElements.dndUploader.find('input').remove();

                                newSlideshowString = self.templateElements.sortableList.find('.ep-dropzone-sortable-item-holder > img').map(function() {
                                    return $(this).data('name');
                                }).get().join(';');

                                slideshowData = o.itemIds.map(function(id) {
                                    return {
                                        id: id,
                                        slideshowString: newSlideshowString
                                    };
                                });

                                self.templateElements.slideshowStrings.val(JSON.stringify(slideshowData));
                            }
                        },
                        Cancel: {
                            cssClass: 'ep-button ep-button-cancel',
                            click: function(){
                                ep(this).uiDialog('close');
                            },
                            text: translations.Cancel
                        }
                    }
                });
            }
        },

        _validateEnvironment: function() {
            var self = this,
                elem = self.element,
                o = self.options;

            // if no item selected or batch action is disabled then abort
            if (o.items().length === 0
                ||
                elem.closest('label').hasClass('BatchActionDisabled')) {
                return false;
            }
            return true;
        },

        _initializeSlideshows: function() {
            var self = this,
                o = self.options;

            o.slideshowArrays[o.itemIds[0]].forEach(function(id) {
                self._selectImageByName(id);
            });
        },

        _initSorting: function(firstChild) {
            var self = this;

            self.templateElements.sortableList.addClass('ep-variationsslideshow-sorting' + ((firstChild === true) ? ' ep-variationsslideshow-firstChildPicked' : ''));
        },

        _stopSorting: function() {
            var self = this;

            self.templateElements.sortableList.removeClass('ep-variationsslideshow-sorting ep-variationsslideshow-firstChildPicked');
        },

        _selectImageById: function(id) {
            var self = this,
                img = self.templateElements.dndUploader.find('img[data-id="' + id + '"]');

            self._selectImageByElement(img);
        },

        _selectImageByName: function(name) {
            var self = this,
                img = self.templateElements.dndUploader.find('img[data-name="' + name + '"]');

            self._selectImageByElement(img);
        },

        _selectImageByElement: function (img) {
            var self = this,
                elem = self.element,
                imgWrapper = img.closest('li.ep-dropzone-sortable-item'),
                listItem = tmplListItem({
                    id: img.data('id'),
                    name: img.data('name')
                });

            if (self.dialog) {
                self.dialog.trigger('change.uiForm');
            }

            if (!imgWrapper.hasClass('selected') && img.length > 0) {
                listItem.tmplItem('elements').imgWrapper.append(img.clone());
                self.templateElements.emptySlideshowItem.before(listItem);

                imgWrapper.addClass('selected');
                imgWrapper.draggable('disable');
            }
        },

        _removeImageById: function(id) {
            var self = this,
                img = self.templateElements.sortableList.find('img[data-id="' + id + '"]');

            if (img && img.length) {
                self._removeImageByElement(img);
            }
        },

        _removeImageByElement: function(img) {
            var self = this,
                elem = self.element,
                $img = $(img),
                imgWrapper = self.templateElements.sortableList.find('li:has(img[data-id="' + $img.data('id') + '"])'),
                listItem = self.templateElements.dndUploader.find('li.selected:has(img[data-id="' + $img.data('id') + '"])');

            if (self.dialog) {
                self.dialog.trigger('change.uiForm');
            }

            listItem.removeClass('selected');
            listItem.draggable('enable');
            imgWrapper.remove();
        },

        _handleSelectedListClick: function(evt) {
            var id = $(evt.target).closest('li').find('img').data('id'),
                self = evt.data.self;

            self._removeImageById(id);
        },

        _handlePoolDrop: function(evt, ui) {
            var self = this,
                elem = self.element,
                imgWrapper = ui.item,
                img = imgWrapper.find('img'),
                imgId = img.data('id'),
                listItem = tmplListItem({
                    id: imgId,
                    name: img.data('name')
                });

            listItem.tmplItem('elements').imgWrapper.append(img.clone());
            self.templateElements.sortableList.find('li:has(img[data-id="' + imgId + '"])').replaceWith(listItem);

            imgWrapper.addClass('selected');
            imgWrapper.draggable('disable');

            if (self.dialog) {
                self.dialog.trigger('change.uiForm');
            }
        },

        _handlePoolClick: function(evt) {
            var self = evt.data.self,
                imgWrapper = $(evt.currentTarget),
                img = imgWrapper.find('img');

            if ($(evt.target).closest('.ep-dropzone-iconholder').length) {
                return false;
            }

            self[!imgWrapper.hasClass('selected') ? '_selectImageByElement' : '_removeImageByElement'](img);
        },

        destroy: function () {
            var self = this;

            self.dialog.uiDialog('destroy');
            self._superApply(arguments);
        }
    });

    return de_epages;

});


